<?php
/*
 * Copyright 2011 BTCinch LLC
 *
 * Released under the GNU General Public License
 *
 * Author: Ira Miller
 * 
 * BTCinch SCI functions, for shopping carts to use in communicationg with the BTCinch payment
 * gateway.
 *
 * You can learn more about these values here: https://btcinch.com/api/docs/BTCinch_SCI.pdf
 */

/**
 * Send request to the payment gateway.
 */
function sendToGateway($parameters) {
  $params = array('VERSION' => BTCINCH_VERSION);

  if (is_array($parameters) && !empty($parameters)) {
    $params = array_merge($params, $parameters);
  }

  if(!array_key_exists('KEY', $params) || empty($params['KEY'])) {
    $params['KEY'] = BTCINCH_KEY;
  }
  if(!array_key_exists('PASS', $params) || empty($params['PASS'])) {
    $params['PASS'] = BTCINCH_PASS;
  }

  $post_string = '';

  foreach ($params as $key => $value) {
    $post_string .= $key . '=' . rawurlencode(trim($value)) . '&';
  }

  $curl = curl_init(BTCINCH_GATEWAY);
  curl_setopt($curl, CURLOPT_RETURNTRANSFER, TRUE);
  curl_setopt($curl, CURLOPT_FORBID_REUSE, TRUE);
  curl_setopt($curl, CURLOPT_FRESH_CONNECT, TRUE);
  curl_setopt($curl, CURLOPT_POST, TRUE);
  curl_setopt($curl, CURLOPT_FOLLOWLOCATION, FALSE);
  curl_setopt($curl, CURLOPT_POSTFIELDS, $post_string);

  $rawresult = curl_exec($curl);

  if(curl_errno($curl)) {
    $results = curl_errno($curl);
  } else {
    $results = json_decode($rawresult);
  }
  curl_close($curl);

  return $results;
}

/**
 * Set details for use in checkout process.
 *
 * Optional arguments:
 * KEY, PASS, OID, TOTAL, CURRENCYCODE, RETURN_URL, CANCEL_URL, TYPE
 *
 * Returns array containing TOKEN to use for this transaction.
 */
function setCheckoutDetails($key = "",
                            $pass = "",
                            $oid = 0,
                            $total = 0,
                            $currencycode = "BTC",
                            $return_url = "",
                            $cancel_url = "",
                            $type = "") {

  $params = array('ACTION' => 'SETDETAILS',
                  'KEY' => $key,
                  'PASS' => $pass,
                  'OID' => $oid,
                  'TOTAL' => $total,
                  'CURRENCYCODE' => $currencycode,
                  'RETURN_URL' => (empty($return_url) ? BTCINCH_RETURN_URL : $return_url),
                  'CANCEL_URL' => (empty($cancel_url) ? BTCINCH_CANCEL_URL : $cancel_url),
                  'TYPE' => (empty($type) ? BTCINCH_TYPE : $type));

  return sendToGateway($params);
}

/**
 * Get details for the transaction associated with the provided token.
 *
 * Required arguments:
 * TOKEN
 *
 * Optional arguments:
 * KEY, PASS
 */
function getCheckoutDetails($key, $pass, $token) {
  if(empty($token)) return false;

  $params = array('ACTION' => 'GETDETAILS',
                  'KEY' => $key,
                  'PASS' => $pass,
                  'TOKEN' => $token);
  return sendToGateway($params);
}

/**
 * Update the Order ID. This should be set to the ID of the current order,
 * in your store DB. Will be used to confirm payment, so make sure this value is correct.
 *
 * Required fields:
 * OID, TOKEN
 *
 * Optional arguments:
 * KEY, PASS
 */
function updateOID($key, $pass, $oid, $token) {
  if(empty($token)) return false;

  $params = array('ACTION' => 'UPDATEOID',
                  'KEY' => $key,
                  'PASS' => $pass,
                  'TOKEN' => $token,
                  'OID' => $oid);
  return sendToGateway($params);
}

/**
 * Update order total and currency code.
 *
 * Required arguments:
 * TOKEN, TOTAL, and CURRENCYCODE
 *
 * Optional arguments:
 * KEY, PASS
 */
function updateTotal($token, $total = 0, $currency = "BTC") {
  if(empty($token)) return false;

  $params = array('ACTION' => 'UPDATETOTAL',
                  'KEY' => $key,
                  'PASS' => $pass,
                  'TOKEN' => $token,
                  'TOTAL' => $total,
                  'CURRENCYCODE' => $currency);

  return sendToGateway($params);
}

/**
 * Get status of existing payment. Use for update verification.
 *
 * Required arguments:
 * OID
 *
 * Optional arguments:
 * KEY, PASS
 */
function getStatus($oid) {
  if(empty($oid)) return false;

  $params = array('ACTION'=>'GETSTATUS',
                  'KEY' => $key,
                  'PASS' => $pass,
                  'OID'=>$oid);
  return sendToGateway($params);
}
?>
